﻿; *******************************************************************************************************************************************************
; AUTHOR        : MicrodevWeb
; PROJECT NAME  : RAD
; MODULE NAME   : DB
; VERSION       : 1
; REQUIERED     : PB 5.60
; *******************************************************************************************************************************************************

DeclareModule DB
  ;======================================================================================================================================================
  ;-* PUBLIC CONSTANTE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Enumeration 
    #Type_Integer
    #Type_Float
    #Type_Double
    #Type_Text
    #Type_Note
    #Type_Euro
    #Type_Percentage
    #Type_Date
    #Type_Hour
    #Type_Phone
    #Type_Email
    #Type_Link
  EndEnumeration
  EnumerationBinary 
    #Not_Null
    #Unique
  EndEnumeration
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC PROTOTYPES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Declare Init(Directory.s,DbName.s,UserName.s,UserPassword.s)
  Declare AddTable(TableName.s,IdName.s)
  Declare AddColumn(ColumnName.s,Label.s,Type.l,Flag.l=0,LinkedTable.s="",LinkedColumn.s="",DiplayedColumn.s="")
  Declare Update(requete.s)
  Declare Build()
  Declare AddCompositeKey(Key.s)
  Declare.s GetLabelColumn(TableName.s,ColumnName.s)
  Declare GetTypeOfColumn(TableName.s,ColumnName.s)
  Declare.s GetLinkedTable(TableName.s,ColumnName.s)
  Declare.s GetLinkedColumn(TableName.s,ColumnName.s)
  Declare.s GetColumnDisplayed(TableName.s,ColumnName.s)
  Declare Query(requete.s)
  Declare.s GetIdTableName(TableName.s)
  Declare GetFlagOfColumn(TableName.s,ColumnName.s)
  Declare Teste()
  Declare GetCompositeKey(TableName.s,List myList.s())
  Declare TableIsLinked(TableName.s,Id)
  Declare DeleteRecord(TableName.s,Id)
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
EndDeclareModule
Module DB
  EnableExplicit
  UseSQLiteDatabase()
  ;======================================================================================================================================================
  ;-* PRIVATE CONSTANTE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE STRUCTURES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Structure sColumn
    Type.l
    Flag.l
    Lable.s
    LinkedTable.s
    LinkedColumn.s
    DiplayedColumn.s
  EndStructure
  Structure sTable
    name.s
    IdName.s
    Map myColumn.sColumn()
    List myCompositeKey.s()
  EndStructure
  Structure sDb
    Directory.s
    DbName.s
    UserName.s
    UserPassword.s
    Map myTable.sTable()
  EndStructure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE MACRO
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Macro mError(FunctionName,Message,ValReturn)
    MessageRequester("DB Error ("+FunctionName+")",Message,#PB_MessageRequester_Error)
    ProcedureReturn ValReturn
  EndMacro
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE VARIABLES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Global myDb.sDb
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE PROTOTYPES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE FUNCTIONS
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Procedure CreateMyFile()
    Protected FileName.s
    With myDb
      ; Création du répertoire
      If FileSize(\Directory)<>-2
        If Not CreateDirectory(\Directory)
          mError("CreateMyFile","Can't create directory "+\Directory,#False)
        EndIf
      EndIf
      FileName=\Directory+"\"+\DbName
      ; Création du fichier
      If FileSize(FileName)=-1
        If Not CreateFile(0,FileName)
          mError("CreateMyFile","Can't create file "+FileName,#False)
        EndIf
        CloseFile(0)
      EndIf
      ProcedureReturn #True
    EndWith
  EndProcedure
  Procedure OpenDb()
    Protected id,FileName.s
    With myDb
      FileName=\Directory+"\"+\DbName
      id=OpenDatabase(#PB_Any,FileName,\UserName,\UserPassword)
      If Not id
        mError("OpenDb","Can't open database",#False)
      EndIf
      ProcedureReturn id
    EndWith
  EndProcedure
  Procedure.s MakeColumn(requete.s)
    Protected n
    With myDb\myTable()\myColumn()
      ForEach myDb\myTable()\myColumn()
        If n<MapSize(myDb\myTable()\myColumn())
          requete+","
        EndIf
        requete+MapKey(myDb\myTable()\myColumn())
        Select \Type
          Case #Type_Integer,#Type_Link
            requete+" INTEGER"
          Case #Type_Float,#Type_Euro,#Type_Percentage
            requete+" FLOAT"
          Case #Type_Date,#Type_Hour,#Type_Text,#Type_Note,#Type_Phone,#Type_Email
            requete+" TEXT"
          Case #Type_Double
            requete+" DOUBLE"
        EndSelect
        n+1
      Next
      requete+")"
      ProcedureReturn requete
    EndWith
  EndProcedure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC FUNCTIONS
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Procedure Init(Directory.s,DbName.s,UserName.s,UserPassword.s)
    With myDb
      \DbName=DbName
      \Directory=Directory
      \UserName=UserName
      \UserPassword=UserPassword
      If Not CreateMyFile()
        ProcedureReturn #False
      EndIf
      ProcedureReturn #True
    EndWith
  EndProcedure
  Procedure AddTable(TableName.s,IdName.s)
    With myDb
      If FindMapElement(\myTable(),TableName)
        mError("AddTable","This table "+TableName+" already exists",#False)
      EndIf
      AddMapElement(\myTable(),TableName)
      \myTable()\IdName=IdName
    EndWith
  EndProcedure
  Procedure AddColumn(ColumnName.s,Label.s,Type.l,Flag.l=0,LinkedTable.s="",LinkedColumn.s="",DiplayedColumn.s="")
    With myDb\myTable()
      If FindMapElement(\myColumn(),ColumnName)
        mError("AddColumn","This column "+ColumnName+" already exists",#False)
      EndIf
      If Type=#Type_Link And LinkedTable=""
        mError("AddColumn","Please fill LinkedTable",#False)
      EndIf
      If Type=#Type_Link And LinkedColumn=""
        mError("AddColumn","Please fill LinkedColumn",#False)
      EndIf
      If Type=#Type_Link And DiplayedColumn=""
        mError("AddColumn","Please fill DiplayedColumn",#False)
      EndIf
      If Type<#Type_Integer Or Type>#Type_Link
        mError("AddColumn","Invalide type of column",#False)
      EndIf
      AddMapElement(\myColumn(),ColumnName)
    EndWith
    With myDb\myTable()\myColumn()
      \Flag=Flag
      \DiplayedColumn=DiplayedColumn
      \LinkedColumn=LinkedColumn
      \LinkedTable=LinkedTable
      \Type=Type
      \Lable=Label
    EndWith
  EndProcedure
  Procedure Update(requete.s)
    Protected idDb=OpenDb()
    If Not idDb
      ProcedureReturn #False
    EndIf
    If Not DatabaseUpdate(idDb,requete)
      CloseDatabase(idDb)
      mError("Update",DatabaseError()+Chr(10)+requete,#False)
    EndIf
    CloseDatabase(idDb)
    ProcedureReturn #True
  EndProcedure
  Procedure Build()
    Protected req.s
    If MapSize(myDb\myTable())=0
      mError("Build","Table of Db is empty",#False)
    EndIf
    ; Vérifier que chaque table possède au minimum une colonne
    ForEach myDb\myTable()
      If MapSize(myDb\myTable()\myColumn())=0
        mError("Build","Column of Table"+MapKey(myDb\myTable())+" is empty",#False)
      EndIf
    Next
    With myDb\myTable()
      ForEach myDb\myTable()
        req="CREATE TABLE IF NOT EXISTS "+MapKey(myDb\myTable())+"("
        req+\IdName+" INTEGER PRIMARY KEY AUTOINCREMENT"
        req=MakeColumn(req)
        If Not Update(req)
          ProcedureReturn #False
        EndIf
      Next
    EndWith
  EndProcedure
  Procedure AddCompositeKey(Key.s)
    AddElement(myDb\myTable()\myCompositeKey())
    myDb\myTable()\myCompositeKey()=Key
  EndProcedure
  Procedure.s GetLabelColumn(TableName.s,ColumnName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetLabelColumn","This table "+TableName+" don't exists","")
      EndIf
      If Not FindMapElement(\myTable()\myColumn(),ColumnName)
        mError("GetLabelColumn","This column "+ColumnName+" don't exists in table "+TableName,"")
      EndIf
      ProcedureReturn  \myTable()\myColumn()\Lable
    EndWith
  EndProcedure
  Procedure GetTypeOfColumn(TableName.s,ColumnName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetTypeOfColumn","This table "+TableName+" don't exists",-1)
      EndIf
      If Not FindMapElement(\myTable()\myColumn(),ColumnName)
        mError("GetTypeOfColumn","This column "+ColumnName+" don't exists",-1)
      EndIf
      ProcedureReturn  \myTable()\myColumn()\Type
    EndWith
  EndProcedure
  Procedure.s GetLinkedTable(TableName.s,ColumnName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetLinkedTable","This table "+TableName+" don't exists","")
      EndIf
      If Not FindMapElement(\myTable()\myColumn(),ColumnName)
        mError("GetLinkedTable","This column "+ColumnName+" don't exists","")
      EndIf
      ProcedureReturn  \myTable()\myColumn()\LinkedTable
    EndWith
  EndProcedure
  Procedure.s GetLinkedColumn(TableName.s,ColumnName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetLinkedColumn","This table "+TableName+" don't exists","")
      EndIf
      If Not FindMapElement(\myTable()\myColumn(),ColumnName)
        mError("GetLinkedColumn","This column "+ColumnName+" don't exists","")
      EndIf
      ProcedureReturn  \myTable()\myColumn()\LinkedColumn
    EndWith
  EndProcedure
  Procedure.s GetColumnDisplayed(TableName.s,ColumnName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetColumnDisplayed","This table "+TableName+" don't exists","")
      EndIf
      If Not FindMapElement(\myTable()\myColumn(),ColumnName)
        mError("GetColumnDisplayed","This column "+ColumnName+" don't exists","")
      EndIf
      ProcedureReturn  \myTable()\myColumn()\DiplayedColumn
    EndWith
  EndProcedure
  Procedure Query(requete.s)
    Protected idDb=OpenDb()
    If Not idDb
      ProcedureReturn #False
    EndIf
    If Not DatabaseQuery(idDb,requete)
      CloseDatabase(idDb)
      mError("Query",DatabaseError()+Chr(10)+requete,#False)
    EndIf
    ProcedureReturn idDb
  EndProcedure
  Procedure.s GetIdTableName(TableName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetIdTableName","This table "+TableName+" don't exists","")
      EndIf
      ProcedureReturn \myTable()\IdName
    EndWith
  EndProcedure
  Procedure GetFlagOfColumn(TableName.s,ColumnName.s)
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetLabelColumn","This table "+TableName+" don't exists",#False)
      EndIf
      If Not FindMapElement(\myTable()\myColumn(),ColumnName)
        mError("GetLabelColumn","This column "+ColumnName+" don't exists",#False)
      EndIf
      ProcedureReturn  \myTable()\myColumn()\Flag
    EndWith
  EndProcedure
  Procedure Teste()
    ForEach myDb\myTable()
      Debug MapKey(myDb\myTable())
      ForEach myDb\myTable()\myColumn()
        Debug MapKey(myDb\myTable()\myColumn())
      Next
      Debug Chr(10)
    Next
  EndProcedure
  Procedure GetCompositeKey(TableName.s,List myList.s())
    Protected Txt.s
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("GetCompositeKey","This table "+TableName+" don't exists",0)
      EndIf
      ForEach \myTable()\myCompositeKey()
        AddElement(myList())
        myList()=\myTable()\myCompositeKey()
      Next
;       Txt=Left(Txt,Len(Txt)-1)
;       ProcedureReturn Txt
    EndWith
  EndProcedure
  Procedure TableIsLinked(TableName.s,Id)
    Protected req.s,idDb
    With myDb
      ForEach \myTable()
        If MapKey(\myTable())<>TableName
          ForEach \myTable()\myColumn() 
            If \myTable()\myColumn()\Type=#Type_Link
              If \myTable()\myColumn()\LinkedTable=TableName
                req="SELECT * FROM "+MapKey(\myTable())+
                    " WHERE "+MapKey(\myTable()\myColumn())+
                    " = "+Id
                idDb=Query(req)
                If Not idDb
                  If IsDatabase(idDb)
                    CloseDatabase(idDb)
                  EndIf
                  ProcedureReturn #True
                EndIf
                If FirstDatabaseRow(idDb)
                  CloseDatabase(idDb)
                  ProcedureReturn #True
                EndIf
              EndIf
            EndIf
          Next
        EndIf
      Next
      ProcedureReturn #False
    EndWith
  EndProcedure
  Procedure DeleteRecord(TableName.s,Id)
    Protected req.s
    With myDb
      If Not FindMapElement(\myTable(),TableName)
        mError("DeleteRecord","This table "+TableName+" don't exists",0)
      EndIf
      req="DELETE FROM "+TableName+
          " WHERE "+\myTable()\IdName+"="+Str(Id)
      If Update(req)
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndWith
  EndProcedure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
EndModule
; IDE Options = PureBasic 5.60 (Windows - x64)
; CursorPosition = 221
; FirstLine = 48
; Folding = NAgIgA-
; EnableXP