﻿; *******************************************************************************************************************************************************
; AUTHOR        : MicrodevWeb
; PROJECT NAME  : RAD
; CLASS NAME    : TabForm
; VERSION       : 1
; REQUIERED     : PB 5.60
; *******************************************************************************************************************************************************
XIncludeFile "TBL.pbi"
DeclareModule TabForm
  ;======================================================================================================================================================
  ;-* PUBLIC CONSTANTE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  EnumerationBinary 
    #SelectMode
  EndEnumeration
  EnumerationBinary 
    #ShearchOn
    #FiltersOn
    #AlignToRight
    #AlignToCenter
  EndEnumeration  
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC INTERFACE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Interface _TabForm
    Open(MotherWindow=-1,Flag.l=0)
    AddColumn(ColumnName.s,Width.f,flag.l)
    SetOrder(OrderClause.s)
  EndInterface
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC PROTOTYPES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Declare New(TableName.s,Title.s,W,H,FicForm)
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
EndDeclareModule
Module TabForm
  EnableExplicit
  UseSQLiteDatabase()
  ;======================================================================================================================================================
  ;-* PRIVATE macro
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Macro mError(FunctionName,Message,ValReturn)
    MessageRequester("FormTab Error ("+FunctionName+")",Message,#PB_MessageRequester_Error)
    ProcedureReturn ValReturn
  EndMacro
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE CONSTANTE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE STRUCTURES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Structure sColumn
    name.s
    flag.l
    width.f
  EndStructure
  Structure sTab
    *VPROC
    idForm.l
    title.s
    flag.l
    FormFlag.l
    minWidth.l
    minHeight.l
    maxHeight.l
    maxWidth.l
    motherWindow.l
    *callBackSelect
    *callBackEdit
    *callBackNew
    TableName.s
    x.l
    y.l
    myTable.TBL::_TBL
    OrderClause.s
    *FicForm.RAD::_FicForm
    List myColumn.sColumn()
  EndStructure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE VARIABLES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Global Font=LoadFont(#PB_Any,"Arial",11,#PB_Font_HighQuality)
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE PROTOTYPES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE FUNCTIONS
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Procedure SelectMyForm()
    Protected *Form
    *Form=GetWindowData(EventWindow())
    ProcedureReturn *Form 
  EndProcedure
  Procedure RefreshMe(*This.sTab)
    With *This
      \myTable\Draw()
    EndWith
  EndProcedure
  Procedure Exit()
    Protected *Form.sTab=SelectMyForm()
    With *Form
      CloseWindow(DialogWindow(\idForm))
      If \motherWindow<>-1
        DisableWindow(\motherWindow,#False)
        SetActiveWindow(\motherWindow)
      EndIf
    EndWith
  EndProcedure
  Procedure EventNew()
    Protected *Form.sTab=SelectMyForm()
    Protected idForm
    With *Form
      idForm=DialogWindow(\idForm)
      \FicForm\Open(idForm,*Form,@RefreshMe())
    EndWith
  EndProcedure
  Procedure Edit()
    Protected *Form.sTab=SelectMyForm()
    Protected idForm,idRecord
    With *Form
      idRecord=\myTable\GetIdSelected()
      If idRecord=-1
        ProcedureReturn 
      EndIf
      idForm=DialogWindow(\idForm)
      \FicForm\Open(idForm,*Form,@RefreshMe(),FicForm::#ModeEdit,idRecord)
    EndWith
  EndProcedure
  Procedure Delete()
    Protected *Form.sTab=SelectMyForm()
    Protected IdRecord
    With *Form
      IdRecord=\myTable\GetIdSelected()
      If IdRecord=-1
        ProcedureReturn 
      EndIf
      If DB::TableIsLinked(\TableName,IdRecord)
        MessageRequester(\title,"Vous ne pouvez-pas supprimer cet enregistrement",#PB_MessageRequester_Warning)
        ProcedureReturn 
      EndIf
      If MessageRequester(\title,"Etes vous-sure de vouloir supprimer cet enregistrement?",#PB_MessageRequester_YesNo|#MB_DEFBUTTON2)=#PB_MessageRequester_No
        ProcedureReturn 
      EndIf
      If DB::DeleteRecord(\TableName,IdRecord)
        \myTable\SetLigneSelected(-1)
        \myTable\Draw()
      EndIf
    EndWith
  EndProcedure
  Procedure Selected()
    Protected *This.sTab=SelectMyForm()
    Protected idRecord
    With *This
      idRecord=\myTable\GetIdSelected()
      If idRecord=-1
        ProcedureReturn 
      EndIf
      If \callBackSelect
        CallFunctionFast(\callBackSelect,idRecord)
      EndIf
      Exit()
    EndWith
  EndProcedure
  Procedure.s GetFormFlag(*this.sTab)
    Protected txt.s
    With *this
      If \FormFlag&#PB_Window_SystemMenu=#PB_Window_SystemMenu
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_SystemMenu"
      EndIf
      If \FormFlag&#PB_Window_ScreenCentered=#PB_Window_ScreenCentered
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_ScreenCentered"
      EndIf
      If \FormFlag&#PB_Window_Maximize=#PB_Window_Maximize
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_Maximize"
      EndIf
      If \FormFlag&#PB_Window_Minimize=#PB_Window_Minimize
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_Minimize"
      EndIf
      If \FormFlag&#PB_Window_MinimizeGadget=#PB_Window_MinimizeGadget
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_MinimizeGadget"
      EndIf 
      If \FormFlag&#PB_Window_MaximizeGadget=#PB_Window_MaximizeGadget
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_MaximizeGadget"
      EndIf 
      If \FormFlag&#PB_Window_SizeGadget=#PB_Window_SizeGadget
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_SizeGadget"
      EndIf 
      If \FormFlag&#PB_Window_SizeGadget=#PB_Window_SizeGadget
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_SizeGadget"
      EndIf
      If \FormFlag&#PB_Window_WindowCentered=#PB_Window_WindowCentered
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_WindowCentered"
      EndIf
      If \FormFlag&#PB_Window_Invisible=#PB_Window_Invisible
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_Invisible"
      EndIf
      If \FormFlag&#PB_Window_BorderLess=#PB_Window_BorderLess
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_BorderLess"
      EndIf
      If \FormFlag&#PB_Window_TitleBar=#PB_Window_TitleBar
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_TitleBar"
      EndIf
      If \FormFlag&#PB_Window_NoGadgets=#PB_Window_NoGadgets
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_NoGadgets"
      EndIf
      If \FormFlag&#PB_Window_NoActivate=#PB_Window_NoActivate
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_NoActivate"
      EndIf
      If \FormFlag&#PB_Window_Invisible=#PB_Window_Invisible
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_Invisible"
      EndIf
      If \motherWindow<>-1
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_windowcentered"
      Else
        If txt<>""
          txt+"|"
        EndIf
        txt+"#PB_Window_SizeGadget"
      EndIf
    EndWith
    ProcedureReturn  txt
  EndProcedure
  Procedure.s MakeXml(*this.sTab)
    Protected myXml.s
    With *this
      myXml="<window name='"+\TableName+"'"
      If \minWidth
        myXml+" width='"+\minWidth+"'"
      Else
        myXml+" width='auto'"
      EndIf
      If \minHeight
        myXml+" height='"+\minHeight+"'"
      Else
        myXml+" height='auto'"
      EndIf
      If \maxWidth
        myXml+" maxwidth='"+\maxWidth+"'" 
      EndIf
      If \maxHeight
        myXml+" maxheight ='"+\maxheight +"'" 
      EndIf
      myXml+" text='"+\title+"'"
      myXml+" flags='"+GetFormFlag(*this)+"'>"
      myXml+Chr(10)
      myXml+"<hbox expand='item:1'>"+Chr(10)
      myXml+"<canvas name='canvas' flags='#PB_Canvas_Container|#PB_Canvas_Keyboard|#PB_Canvas_Border'/>"+Chr(10)
      myXml+"<vbox expand='no'>"+Chr(10)
      myXml+"<button name='bt_new' text='Nouveau'/>"+Chr(10)
      myXml+"<button name='bt_edit' text='Editer'/>"+Chr(10)
      myXml+"<button name='bt_delete' text='Supprimer'/>"+Chr(10)
      If \flag&#SelectMode
        myXml+"<button name='bt_selected' text='Selectionner'/>"+Chr(10)
      EndIf
      myXml+"<button name='bt_exit' text='Quitter'/>"+Chr(10)
      myXml+"</vbox>"+Chr(10)
      myXml+"</hbox>"+Chr(10)
      myXml+"</window>"+Chr(10)
      ;       CreateFile(0,"teste.xml")
      ;       WriteString(0,myXml)
      ;       CloseFile(0)
      ProcedureReturn myXml
    EndWith
  EndProcedure
  Procedure MakeEventGesture(*this.sTab)
    With *this
      BindEvent(#PB_Event_CloseWindow,@Exit(),DialogWindow(\idForm))
      BindGadgetEvent(DialogGadget(\idForm,"bt_exit"),@Exit())
      BindGadgetEvent(DialogGadget(\idForm,"bt_new"),@EventNew())
      BindGadgetEvent(DialogGadget(\idForm,"bt_edit"),@Edit())
      BindGadgetEvent(DialogGadget(\idForm,"bt_delete"),@Delete())
      ;Ajout des raccourci clavier
      AddKeyboardShortcut(DialogWindow(\idForm),#PB_Shortcut_Insert,0)
      AddKeyboardShortcut(DialogWindow(\idForm),#PB_Shortcut_F4,1)
      AddKeyboardShortcut(DialogWindow(\idForm),#PB_Shortcut_Delete,2)
      AddKeyboardShortcut(DialogWindow(\idForm),#PB_Shortcut_Escape,3)
      BindEvent(#PB_Event_Menu,@Exit(),DialogWindow(\idForm),3)
      BindEvent(#PB_Event_Menu,@EventNew(),DialogWindow(\idForm),0)
      BindEvent(#PB_Event_Menu,@Edit(),DialogWindow(\idForm),1)
      If IsGadget(DialogGadget(\idForm,"bt_selected"))
        GadgetToolTip(DialogGadget(\idForm,"bt_selected"),"Sélectionner [ENTER]")
        AddKeyboardShortcut(DialogWindow(\idForm),#PB_Shortcut_Return,4)
        BindEvent(#PB_Event_Menu,@Selected(),DialogWindow(\idForm),4)
        BindGadgetEvent(DialogGadget(\idForm,"bt_selected"),@Selected())
      EndIf
    EndWith
  EndProcedure
  Procedure CreateMyTable(*this.sTab)
    Protected IdCanvas
    With *this
      IdCanvas=DialogGadget(\idForm,"canvas")
      \myTable=TBL::New(IdCanvas,\TableName)
      ForEach \myColumn()
        \myTable\AddColumn(\myColumn()\name,\myColumn()\width,\myColumn()\flag)
      Next
      If \OrderClause<>""
        \myTable\SetOrder(" ORDER BY "+\OrderClause)
      EndIf
      \myTable\Draw()
    EndWith
  EndProcedure
  
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC FUNCTIONS
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Procedure New(TableName.s,Title.s,W,H,FicForm)
    Protected *Form.sTab
    *Form=AllocateStructure(sTab)
    With *Form
      \TableName=TableName
      \title=Title
      \FicForm=FicForm
      \FormFlag=#PB_Window_SystemMenu|#PB_Window_SizeGadget|#PB_Window_MaximizeGadget
      \VPROC=?VPROC
      \minWidth=W
      \minHeight=H
      ProcedureReturn *Form
    EndWith
  EndProcedure
  Procedure Open(*This.sTab,MotherWindow=-1,Flag.l=0,*Callback=-1)
    Protected xml.s,myXml
    With *This
      \motherWindow=MotherWindow
      \flag=Flag
      \callBackSelect=*Callback
      SetGadgetFont(#PB_Default,FontID(Font))
      xml=MakeXml(*this)
      myXml=CatchXML(#PB_Any, @xml, StringByteLength(xml), 0,#PB_UTF8)
      If Not myXml Or XMLStatus(myXml) <> #PB_XML_Success
        mError("Open",XMLError(myXml)+" line "+Str(XMLErrorLine(myXml)),#False)
      EndIf
      \IdForm=CreateDialog(#PB_Any)
      If Not \IdForm
        mError("Open","Can't create dialog",#False)
      EndIf
      If \motherWindow<>-1
        DisableWindow(\motherWindow,#True)
        If Not OpenXMLDialog(\IdForm,myXml,\TableName,\x,\y,0,0,WindowID(\motherWindow))
          mError("Open","Can't open dialog",#False)
        EndIf 
      Else
        If Not OpenXMLDialog(\IdForm,myXml,\TableName,\x,\y)
          mError("Open","Can't open dialog",#False)
        EndIf
      EndIf
      SetWindowData(DialogWindow(\idForm),*This)
      MakeEventGesture(*This)
      CreateMyTable(*This)
    EndWith
  EndProcedure
  Procedure AddColumn(*This.sTab,ColumnName.s,Width.f,flag.l)
    With *This  
      AddElement(\myColumn())
      \myColumn()\name=ColumnName
      \myColumn()\width=Width
      \myColumn()\flag=flag
    EndWith
  EndProcedure
  Procedure SetOrder(*This.sTab,OrderClause.s)
    With *This
      \OrderClause=OrderClause
    EndWith
  EndProcedure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  DataSection
    VPROC:
    Data.i @Open()
    Data.i @AddColumn()
    Data.i @SetOrder()
  EndDataSection
EndModule
; IDE Options = PureBasic 5.60 (Windows - x64)
; CursorPosition = 424
; FirstLine = 21
; Folding = QACAi-
; EnableXP